@@ -20,11 +20,7 @@ VIRT_PERF_THRESHOLD_MIN=160
 
 do_test()
 {
-	if [ -z $ip_local -o -z $ip_remote ]; then
-		tst_brk TBROK "you must specify IP address"
-	fi
-
-	virt_check_cmd virt_add ltp_v0 id 0 $2 || continue
+	virt_check_cmd virt_add ltp_v0 id 0 $2 || return
 
 	tst_res TINFO "the same VNI must work"
 	# VNI is 24 bits long, so max value, which is not reserved, is 0xFFFFFE
