@@ -10,6 +10,7 @@ from os_tests import tests
 import json
 import difflib
 import time
+import logging
 from tipset.libs import rmt_ssh
 from yaml import load, dump
 try:
@@ -17,32 +18,47 @@ try:
 except ImportError:
     from yaml import Loader, Dumper
 
-def init_connection(test_instance, timeout=600):
-    if test_instance.params['remote_node'] is None:
-        return
-
-    test_instance.log.info("remote_node specified, all tests will be run in {}".format(test_instance.params['remote_node']))
+def init_ssh(params=None, timeout=600, log=None):
+    if log is None:
+        LOG_FORMAT = '%(levelname)s:%(message)s'
+        log = logging.getLogger(__name__)
+        logging.basicConfig(level=logging.INFO, format=LOG_FORMAT)
     ssh = rmt_ssh.RemoteSSH()
-    ssh.rmt_node = test_instance.params['remote_node']
-    ssh.rmt_user = test_instance.params['remote_user']
-    ssh.rmt_password = test_instance.params['remote_password']
-    ssh.rmt_keyfile = test_instance.params['remote_keyfile']
-
-    ssh.timeout = 180
-    ssh.log = test_instance.log
-    test_instance.SSH = ssh
+    ssh.rmt_node = params['remote_node']
+    ssh.rmt_user = params['remote_user']
+    ssh.rmt_password = params['remote_password']
+    ssh.rmt_keyfile = params['remote_keyfile']
+    ssh.log = log
     start_time = time.time()
     while True:
         current_time = time.time()
         if current_time - start_time > timeout:
-            test_instance.log.info("timeout to connect to remote")
-            break
-        test_instance.SSH.create_connection()
-        if test_instance.SSH.ssh_client is not None:
-            break
+            log.info("timeout to connect to remote")
+            return None
+        ssh.create_connection()
+        if ssh.ssh_client is not None:
+            return ssh
         time.sleep(5)
-        test_instance.log.info("Not conncted, retry again! timeout:{}".format(timeout))
-    if test_instance.SSH.ssh_client is None:
+        log.info("Not conncted, retry again! timeout:{}".format(timeout))
+    return None
+
+def init_connection(test_instance, timeout=600):
+    if test_instance.params['remote_node'] is None:
+        return
+    test_instance.log.info("remote_node specified, all tests will be run in {}".format(test_instance.params['remote_node']))
+    try:
+        if hasattr(test_instance.SSH, 'ssh_client'):
+            test_instance.SSH.log = test_instance.log
+            ret, _, _ = test_instance.SSH.cli_run(cmd='uname -r')
+            if ret == 0:
+                test_instance.log.info("connection is live")
+                return
+    except AttributeError:
+        pass
+    except Exception:
+        test_instance.log.info("connection is not live")
+    test_instance.SSH = init_ssh(params=test_instance.params, timeout=timeout, log=test_instance.log)
+    if test_instance.SSH is None:
         test_instance.skipTest("Cannot make ssh connection to remote, please check")
 
 def get_cfg():
@@ -60,6 +76,9 @@ def init_case(test_instance):
     Arguments:
         test_instance {Test instance} -- unittest.TestCase instance
     """
+    if not hasattr(test_instance,'params'):
+        cfg_file, keys_data = get_cfg()
+        test_instance.params = keys_data
     results_dir = test_instance.params['results_dir']
     debug_dir = results_dir + "/debug"
     test_instance.log_dir = results_dir
@@ -76,7 +95,7 @@ def init_case(test_instance):
         handler.close()
         logging.root.removeHandler(handler)
     FORMAT = "%(levelname)s:%(message)s"
-    logging.basicConfig(level=logging.DEBUG, format=FORMAT, filename=log_file)
+    logging.basicConfig(level=logging.INFO, format=FORMAT, filename=log_file)
     test_instance.log.info("-"*80)
     test_instance.log.info("Code Repo: {}".format(test_instance.params['code_repo']))
     test_instance.log.info("Case ID: {}".format(test_instance.id()))
@@ -90,11 +109,20 @@ def init_case(test_instance):
     test_instance.log.info("-"*80)
     if test_instance.params['remote_node'] is not None:
         init_connection(test_instance)
+        test_instance.SSH.log = test_instance.log
         if  test_instance.SSH.ssh_client is None:
             test_instance.skipTest("Cannot make ssh connection to remote, please check")
+    node_info = "{}/node_info".format(debug_dir)
+    if not os.path.exists(node_info):
+        test_instance.log.info("retrive node info.")
+        release_name = get_os_release_info(test_instance, field="NAME")
+        release_version = get_os_release_info(test_instance, field="VERSION")
+        kernel_version = run_cmd(test_instance, 'uname -r')
+        with open(node_info, 'w+') as fh:
+            fh.write("{} {} - {}".format(release_name, release_version, kernel_version))
 
 def finish_case(test_instance):
-    """init case
+    """finish case
     Arguments:
         test_instance {Test instance} -- unittest.TestCase instance
     """
@@ -195,7 +223,7 @@ def run_cmd(test_instance,
         test_cmd = 'uname -a'
         test_instance.log.info("Test system is alive via cmd:{}. If still fail, check no hang or panic happens.".format(test_cmd))
         try:
-            if test_instance.SSH.ssh_client is not None:
+            if test_instance.params['remote_node'] is not None:
                 status, output = test_instance.SSH.remote_excute(test_cmd, timeout)
                 status, output = test_instance.SSH.remote_excute(cmd, timeout, redirect_stdout=rmt_redirect_stdout, redirect_stderr=rmt_redirect_stderr,rmt_get_pty=rmt_get_pty)
             else:
@@ -318,22 +346,22 @@ def getboottime(test_instance):
     Arguments:
         test_instance {Test instance} -- unittest.TestCase instance
     '''
-    run_cmd(test_instance, "sudo which systemd-analyze", expect_ret=0)
+    run_cmd(test_instance, "which systemd-analyze", expect_ret=0)
     time_start = int(time.time())
     while True:
-        output = run_cmd(test_instance, "sudo systemd-analyze")
+        output = run_cmd(test_instance, "systemd-analyze")
         if 'Bootup is not yet finished' not in output:
             break
         time_end = int(time.time())
-        run_cmd(test_instance, 'sudo systemctl list-jobs')
+        run_cmd(test_instance, 'systemctl list-jobs')
         if time_end - time_start > 60:
             test_instance.fail("Bootup is not yet finished after 60s")
         test_instance.log.info("Wait for bootup finish......")
         time.sleep(1)
-    cmd = "sudo systemd-analyze blame > /tmp/blame.log"
+    cmd = "systemd-analyze blame > /tmp/blame.log"
     run_cmd(test_instance, cmd, expect_ret=0)
     run_cmd(test_instance, "cat /tmp/blame.log", expect_ret=0)
-    output = run_cmd(test_instance, "sudo systemd-analyze", expect_ret=0)
+    output = run_cmd(test_instance, "systemd-analyze", expect_ret=0)
     boot_time = re.findall("=.*s", output)[0]
     boot_time = boot_time.strip("=\n")
     boot_time_sec = re.findall('[0-9.]+s', boot_time)[0]
@@ -397,7 +425,9 @@ def is_azure(test_instance, action=None):
         azure: return True
         other: return False
     '''
-    output = run_cmd(test_instance, "sudo route -n", expect_ret=0)
+    output = run_cmd(test_instance, "route -n")
+    if 'not found' in output:
+        return False
     if '168.63.129.16' in output:
         test_instance.log.info("Azure system.")
         return True
@@ -496,7 +526,7 @@ def is_cmd_exist(test_instance, cmd=None, is_install=True, cancel_case=False):
     run_cmd(test_instance, "sudo yum install -y {}".format(pkg_name), expect_ret=0, timeout=180)
     return True
 
-def is_pkg_installed(test_instance, pkg_name=None, is_install=True, cancel_case=False):
+def is_pkg_installed(test_instance, pkg_name=None, is_install=True, cancel_case=False, timeout=120):
     '''
     check cmd exists status, if no, try to install it.
     Arguments:
@@ -512,12 +542,12 @@ def is_pkg_installed(test_instance, pkg_name=None, is_install=True, cancel_case=
         test_instance.log.info("No %s found!" % pkg_name)
         if is_install:
             cmd = 'sudo yum install -y {}'.format(pkg_name)
-            ret = run_cmd(test_instance, cmd, ret_status=True, msg='try to install it')
+            ret = run_cmd(test_instance, cmd, ret_status=True, msg='try to install it', timeout=timeout)
             if ret == 0:
                 return True
         return False
 
-def pkg_install(test_instance, pkg_name=None, pkg_url=None):
+def pkg_install(test_instance, pkg_name=None, pkg_url=None, force=False):
         """
         Install pkg in target system from default repo or pkg_url.
         $pkg_url_$arch is defined in configuration file.
@@ -531,7 +561,6 @@ def pkg_install(test_instance, pkg_name=None, pkg_url=None):
             pkg_name {string} -- pkg name
             pkg_url {string} -- pkg url or location if it is not in default repo
         """
-
         if not is_pkg_installed(test_instance, pkg_name=pkg_name):
             test_instance.log.info("Try install {} automatically!".format(pkg_name))
             if pkg_url is not None:
@@ -551,9 +580,11 @@ def pkg_install(test_instance, pkg_name=None, pkg_url=None):
                 cmd = 'sudo yum -y reinstall %s' % pkg_name
             run_cmd(test_instance, cmd, timeout=1200)
 
-        if not is_pkg_installed(test_instance, pkg_name=pkg_name) and pkg_url is not None:
+        if not is_pkg_installed(test_instance, pkg_name=pkg_name) and pkg_url is not None or force:
             test_instance.log.info('Install without dependences!')
             cmd = 'sudo rpm -ivh %s --nodeps' % pkg_url
+            if force:
+                cmd = cmd + " --force"
             run_cmd(test_instance, cmd, timeout=1200)
         if not is_pkg_installed(test_instance, pkg_name=pkg_name):
             test_instance.skipTest("Cannot install {} automatically!".format(pkg_name))
@@ -575,7 +606,7 @@ def get_memsize(test_instance, action=None):
     test_instance.log.info("Total memory: {:0,.1f}GiB".format(mem_gb))
     return mem_gb
 
-def get_cmd_cursor(test_instance, cmd='dmesg -T', rmt_redirect_stdout=False, rmt_get_pty=False):
+def get_cmd_cursor(test_instance, cmd='dmesg -T', rmt_redirect_stdout=False, rmt_get_pty=False, timeout=60):
     '''
     Get command cursor by last matched line.
     Arguments:
@@ -583,7 +614,7 @@ def get_cmd_cursor(test_instance, cmd='dmesg -T', rmt_redirect_stdout=False, rmt
     Return:
         cursor {string}
     '''
-    output = run_cmd(test_instance, cmd, expect_ret=0, is_log_output=False, rmt_redirect_stdout=rmt_redirect_stdout, rmt_get_pty=rmt_get_pty)
+    output = run_cmd(test_instance, cmd, expect_ret=0, is_log_output=False, rmt_redirect_stdout=rmt_redirect_stdout, rmt_get_pty=rmt_get_pty, timeout=timeout)
     if len(output.split('\n')) < 5:
         return output.split('\n')[-1]
     for i in range(-1, -10, -1):
@@ -617,6 +648,8 @@ def check_log(test_instance, log_keyword, log_cmd="journalctl -b 0", match_word_
 
     if match_word_exact:
         check_cmd = check_cmd + '|grep -iw %s' % log_keyword
+    if skip_words:
+        check_cmd = check_cmd + '|grep -Ev "%{}"'.format(skip_words.replace(',','|'))
     ret = False
     if cursor is not None:
         out = run_cmd(test_instance,
@@ -795,4 +828,18 @@ def get_os_release_info(test_instance, field="VERSION_ID"):
     cmd = "source {} ;echo ${}".format(data_file, field)
     output = run_cmd(test_instance,cmd, expect_ret=0, msg='get {} from {}'.format(field, data_file))
     test_instance.log.info("Got: {}".format(output))
-    return output
\ No newline at end of file
+    return output.strip('\n')
+
+def set_service(test_instance, service="systemd-journald-audit.socket", enable_it = False, check_ret = False):
+    if not enable_it:
+        cmd_dict = [{'cmd':'sudo systemctl stop {}'.format(service), 'msg':'stop {}'.format(service)},
+                {'cmd':'sudo systemctl disable {}'.format(service), 'msg':'disable {}'.format(service)},
+                {'cmd':'sudo systemctl mask {}'.format(service), 'msg':'mask {} to prevent others bring it up'.format(service)}]
+    else:
+        cmd_dict = [{'cmd':'sudo systemctl unmask {}'.format(service), 'msg':'unmask {}'.format(service)},
+                {'cmd':'sudo systemctl enable --now {}'.format(service), 'msg':'enable and start {}'.format(service)}]
+    for cmd in cmd_dict:
+        if check_ret:
+            run_cmd(test_instance,cmd['cmd'], expect_ret=0, msg=cmd['msg'])
+        else:
+            run_cmd(test_instance,cmd['cmd'], msg=cmd['msg'])
\ No newline at end of file
