@@ -8,6 +8,35 @@ import time
 class TestGeneralCheck(unittest.TestCase):
     def setUp(self):
         utils_lib.init_case(self)
+        if self.id().endswith(('test_check_rpm_V_differences', 'test_check_rpm_V_missing')):
+            rpm_V_file = '/tmp/{}_rpm_V.log'.format(self.run_uuid)
+            self.output = utils_lib.run_cmd(self, 'cat {}'.format(rpm_V_file), msg="check if output exists")
+            if 'No such file' in self.output:
+                skips = ['rhui','ltp']
+                pkg = utils_lib.run_cmd(self, 'rpm -qa', msg="list the pkgs")
+                pkgs = pkg.split('\n')
+                import copy
+                check_list = copy.deepcopy(pkgs)
+                for pkg in pkgs:
+                    for i in skips:
+                        if i in pkg:
+                            check_list.remove(pkg)
+                cmd = 'sudo rpm -V {} > {} 2>&1'.format(' '.join(check_list), rpm_V_file)
+                utils_lib.run_cmd(self, cmd, msg="verify pkgs", timeout=600)
+                self.output = utils_lib.run_cmd(self, 'cat {}'.format(rpm_V_file), expect_ret=0, msg="check if output exists again")
+        if self.id().endswith(('test_check_systemd_analyze_verify_missing', 'test_check_systemd_analyze_verify_deprecated_unsafe', 'test_check_systemd_analyze_verify_obsolete', 'test_check_systemd_analyze_verify_ordering_cycle')):
+            self.systemd_analyze_verify_file = '/tmp/{}_systemd_analyze_verify.log'.format(self.run_uuid)
+            output = utils_lib.run_cmd(self, 'cat {}'.format(self.systemd_analyze_verify_file), msg="check if output exists")
+            if 'No such file' in output:
+                services = ['default.target']
+                cmd = "systemctl list-unit-files |grep -v UNIT|grep -v listed|awk -F' ' '{print $1}'"
+                all_services = utils_lib.run_cmd(self, cmd, msg='retrive all systemd unit files').split('\n')
+                for service in all_services:
+                    if not service or service.startswith('-'):
+                        continue
+                    services.append(service)
+                cmd = "sudo systemd-analyze verify {} > {} 2>&1".format(' '.join(services),self.systemd_analyze_verify_file)
+                utils_lib.run_cmd(self, cmd, msg='verify services')
 
     def test_check_avclog(self):
         '''
@@ -25,11 +54,11 @@ class TestGeneralCheck(unittest.TestCase):
         self.log.info("Check no permission denied at nfs server - bug1655493")
         utils_lib.is_pkg_installed(self,pkg_name='nfs-utils')
         output = utils_lib.run_cmd(self, 'uname -r', expect_ret=0)
-
-        if 'el7' in output or 'el6' in output:
-            cmd = "sudo systemctl start nfs"
-        else:
+        product_id = utils_lib.get_os_release_info(self, field='VERSION_ID')
+        if float(product_id) >= 8.0:
             cmd = 'sudo systemctl start nfs-server.service'
+        else:
+            cmd = "sudo systemctl start nfs"
 
         utils_lib.run_cmd(self, cmd, expect_ret=0)
         utils_lib.run_cmd(self, "sudo mkdir /tmp/testrw")
@@ -622,21 +651,15 @@ itlb_multihit|sed 's/:/^/' | column -t -s^"
             - service unit file content
             - output from 'systemd-analyze verify $service'
         """
-
+        services = ['default.target']
         cmd = "systemctl list-unit-files |grep -v UNIT|grep -v listed|awk -F' ' '{print $1}'"
-        all_services = utils_lib.run_cmd(self, cmd, msg='Get all systemd unit files').split('\n')
-
+        all_services = utils_lib.run_cmd(self, cmd, msg='retrive all systemd unit files').split('\n')
         for service in all_services:
-            if len(service) == 0:
-                continue
-            cmd = "systemctl status {}".format(service)
-            utils_lib.run_cmd(self, cmd)
-            cmd = "journalctl --unit {}".format(service)
-            utils_lib.check_log(self,'Unknown lvalue', log_cmd=cmd, rmt_redirect_stdout=True)
             if not service or service.startswith('-'):
                 continue
-            cmd = "sudo systemd-analyze verify {}".format(service)
-            utils_lib.run_cmd(self, cmd, expect_not_kw='Unknown lvalue', msg='Check there is no Unknown lvalue in {}'.format(service))
+            services.append(service)
+        cmd = "sudo systemd-analyze verify {}".format(' '.join(services))
+        utils_lib.run_cmd(self, cmd, expect_not_kw='Unknown lvalue', msg='Check there is no Unknown lvalue')
 
     def test_check_locale(self):
         '''
@@ -907,114 +930,6 @@ itlb_multihit|sed 's/:/^/' | column -t -s^"
                         expect_not_kw='microcode updated early',
                         msg='microcode should not load in VMs')
 
-    def test_check_nitro_enclaves(self):
-        '''
-        case_name:
-            test_check_nitro_enclaves
-        case_priority:
-            1
-        component:
-            kernel
-        bugzilla_id:
-            2011739
-        polarion_id:
-            n/a
-        maintainer:
-            xiliang@redhat.com
-        description:
-            Test whether nitro enclave is enabled.
-        key_steps:
-            1.$ sudo dnf groupinstall "Development Tools"
-            2.$ for version under 9 add centos docker repo
-                and for rhel9 add fedora34 docker repo
-            3.$ sudo dnf install docker-ce docker-ce-cli containerd.io -y
-            4.$ sudo systemctl start docker
-            5.$ sudo systemctl enable docker
-            6.$ sudo usermod -aG docker $USER and re-login
-            7.$ git clone https://github.com/aws/aws-nitro-enclaves-cli.git
-            8.$ cd aws-nitro-enclaves-cli/ and change bootstrap/nitro-cli-config,bootstrap/env.sh,Makefile
-            9.$export NITRO_CLI_INSTALL_DIR=/
-            10.$make nitro-cli
-            11.$make vsock-proxy
-            12.$sudo make NITRO_CLI_INSTALL_DIR=/ install
-            13.$source /etc/profile.d/nitro-cli-env.sh
-            14.$echo source /etc/profile.d/nitro-cli-env.sh >> ~/.bashrc
-            15.$nitro-cli-config -i
-            16.$sudo systemctl enable nitro-enclaves-allocator.service && sudo systemctl start nitro-enclaves-allocator.service
-            17.$nitro-cli build-enclave --docker-dir /usr/share/nitro_enclaves/examples/hello --docker-uri hello:latest --output-file hello.eif
-            18.$nitro-cli run-enclave --cpu-count 2 --memory 512 --enclave-cid 16 --eif-path hello.eif --debug-mode
-            19.$nitro-cli describe-enclaves
-            20.$nitro-cli console --enclave-id $EnclaveID
-            21.$nitro-cli terminate-enclave --enclave-id $EnclaveID
-        expected_result:
-            Enclave can be started and terminated successfully.
-        '''
-        if not utils_lib.is_aws(self):
-            self.skipTest('encalve is only for aws platform')
-        product_name = utils_lib.get_os_release_info(self, field='NAME')
-        if 'Red Hat Enterprise Linux' not in product_name:
-            self.skipTest('Only support run in RHEL for now.')
-        update_files = ["aws-nitro-enclaves-cli/bootstrap/nitro-cli-config", "aws-nitro-enclaves-cli/bootstrap/env.sh",
-                        "aws-nitro-enclaves-cli/Makefile"]
-        first = ["# Remove an older driver if it is inserted.", "lsmod | grep -q nitro_enclaves || ",
-                 "install: install-tools nitro_enclaves"]
-        end = ['[ "$(lsmod | grep -cw $DRIVER_NAME)" -eq 1 ] || fail "The driver is not visible."',
-               'sudo insmod ${NITRO_CLI_INSTALL_DIR}/lib/modules/extra/nitro_enclaves/nitro_enclaves.ko',
-               '${NITRO_CLI_INSTALL_DIR}/lib/modules/$(uname -r)/extra/nitro_enclaves/nitro_enclaves.ko']
-        utils_lib.run_cmd(self, 'sudo dnf groupinstall "Development Tools" -y', msg='install development tools')
-        product_id = utils_lib.get_product_id(self)
-        if float(product_id) < 9:
-            utils_lib.run_cmd(self,
-                              'sudo dnf config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo',
-                              msg='add docker repo')
-            utils_lib.run_cmd(self, 'sudo dnf install docker-ce docker-ce-cli containerd.io --allowerasing -y',
-                              msg='install docker')
-        else:
-            utils_lib.run_cmd(self,
-                              'echo -e "[fedora]\nname=Fedora 34 - \$basearch\nbaseurl=https://download-ib01.fedoraproject.org/pub/fedora/linux/releases/34/Everything/\$basearch/os/" |sudo tee /etc/yum.repos.d/fedora34.repo')
-            utils_lib.run_cmd(self,
-                              'sudo dnf config-manager --add-repo https://download.docker.com/linux/fedora/docker-ce.repo',
-                              msg='add docker repo')
-            utils_lib.run_cmd(self, "sudo sed -i 's/$releasever/34/g' /etc/yum.repos.d/docker-ce.repo")
-            utils_lib.run_cmd(self, 'sudo dnf module disable container-tools -y')
-            utils_lib.run_cmd(self,
-                              'sudo dnf install docker-ce docker-ce-cli containerd.io --nogpgcheck --allowerasing -y',
-                              msg='install docker')
-            utils_lib.run_cmd(self, 'sudo dnf module enable container-tools -y')
-            utils_lib.run_cmd(self, 'sudo dnf config-manager --disable fedora')
-        utils_lib.run_cmd(self, 'sudo systemctl start docker && sudo systemctl enable docker',
-                          msg='enable and start docker')
-        utils_lib.run_cmd(self, 'sudo usermod -aG docker $USER', msg='add to the docker group')
-        utils_lib.run_cmd(self, 'sudo chmod a+rw /var/run/docker.sock')
-        utils_lib.run_cmd(self, 'git clone https://github.com/aws/aws-nitro-enclaves-cli.git',
-                          msg='clone nitro-enclaves-cli ')
-        for i in range(3):
-            first_row = int(utils_lib.run_cmd(self, f"grep -n '{first[i]}' {update_files[i]} | cut -f1 -d:"))
-            end_row = int(utils_lib.run_cmd(self, f"grep -n '{end[i]}' {update_files[i]} | cut -f1 -d:"))
-            utils_lib.run_cmd(self, f"sed -i '{first_row}, {end_row}d' {update_files[i]}")
-        utils_lib.run_cmd(self, f"sed -i '{first_row}i\install: install-tools' aws-nitro-enclaves-cli/Makefile ")
-        utils_lib.run_cmd(self,
-                          'cd aws-nitro-enclaves-cli ; export NITRO_CLI_INSTALL_DIR=/ && make nitro-cli && make vsock-proxy;sudo make NITRO_CLI_INSTALL_DIR=/ install',
-                          timeout=1200, msg='make and install nitro-cli')
-        utils_lib.run_cmd(self, 'echo source /etc/profile.d/nitro-cli-env.sh >> ~/.bashrc')
-        utils_lib.run_cmd(self,
-                          'source /etc/profile.d/nitro-cli-env.sh ; cd aws-nitro-enclaves-cli ; timeout 5 nitro-cli-config -i',
-                          expect_not_kw='Could not')
-        utils_lib.run_cmd(self,
-                          'sudo systemctl enable nitro-enclaves-allocator.service && sudo systemctl start nitro-enclaves-allocator.service')
-        utils_lib.run_cmd(self,
-                          'nitro-cli build-enclave --docker-dir /usr/share/nitro_enclaves/examples/hello --docker-uri hello:latest --output-file hello.eif',
-                          msg='build enclave')
-        utils_lib.run_cmd(self,
-                          'sudo nitro-cli run-enclave --cpu-count 2 --memory 512 --enclave-cid 16 --eif-path hello.eif --debug-mode',
-                          expect_kw='Started', msg='run enclave')
-        EnclaveID = utils_lib.run_cmd(self, 'nitro-cli describe-enclaves |grep EnclaveID', msg='get EnclaveID')
-        EnclaveID = EnclaveID[18:-3]
-        utils_lib.run_cmd(self, f'timeout 10 nitro-cli console --enclave-id {EnclaveID}', expect_kw='Successfully',
-                          msg='get the console')
-        utils_lib.run_cmd(self, f'sudo nitro-cli terminate-enclave --enclave-id {EnclaveID}',
-                          expect_kw='"Terminated": true', msg='terminate enclave')
-
     def test_check_nouveau(self):
         '''
         polarion_id: N/A
@@ -1025,12 +940,12 @@ itlb_multihit|sed 's/:/^/' | column -t -s^"
         self.log.info("nouveau is not required in ec2, make sure it is \
 in blacklist and not loaded bug1645772")
         utils_lib.run_cmd(self,
-                    "sudo lsmod",
+                    "lsmod",
                     expect_ret=0,
                     expect_not_kw="nouveau",
                     msg="Checking lsmod")
         utils_lib.run_cmd(self,
-                    "sudo cat /proc/cmdline",
+                    "cat /proc/cmdline",
                     expect_ret=0,
                     expect_kw="rd.blacklist=nouveau",
                     msg="Checking cmdline")
@@ -1057,7 +972,7 @@ in cmdline as bug1859088")
         polarion_id: RHEL7-103850
         bz: 1852657
         '''
-        check_cmd = "sudo cat /etc/redhat-release"
+        check_cmd = "cat /etc/redhat-release"
         output = utils_lib.run_cmd(self,check_cmd, expect_ret=0, msg='check release name')
         kernel_ver = utils_lib.run_cmd(self, 'uname -r', msg="Get kernel version")
         name_map = {'el6':'Red Hat Enterprise Linux Server release 6',
@@ -1114,11 +1029,11 @@ in cmdline as bug1859088")
         issue: RHELPLAN-60817
         check if product id matches /etc/redhat-release
         '''
-        check_cmd = "sudo cat /etc/redhat-release"
+        check_cmd = "cat /etc/redhat-release"
         output = utils_lib.run_cmd(self,check_cmd, expect_ret=0, cancel_not_kw='CentOS,Fedora', msg='check release name')
         product_id = re.findall('\d.\d', output)[0]
         self.log.info("Get product id: {}".format(product_id))
-        cmd = 'sudo rpm -qa|grep redhat-release'
+        cmd = 'rpm -qa|grep redhat-release'
         utils_lib.run_cmd(self,cmd, cancel_ret='0', msg='get redhat-release-server version')
         cmd = 'sudo rct cat-cert /etc/pki/product-default/*.pem'
         utils_lib.run_cmd(self,cmd, expect_ret=0, expect_kw="Version: {}".format(product_id), msg='check product certificate')
@@ -1219,16 +1134,7 @@ in cmdline as bug1859088")
         expected_result:
             No 'deprecated' or 'unsafe' found in output
         '''
-        cmd = "sudo systemd-analyze verify default.target"
-        utils_lib.run_cmd(self, cmd, expect_not_kw='deprecated,nsafe', msg='Check there is no "deprecated" or "unsafe" keyword in output')
-        cmd = "systemctl list-unit-files |grep -v UNIT|grep -v listed|awk -F' ' '{print $1}'"
-        all_services = utils_lib.run_cmd(self, cmd, msg='retrive all systemd unit files').split('\n')
-
-        for service in all_services:
-            if not service or service.startswith('-'):
-                continue
-            cmd = "sudo systemd-analyze verify {}".format(service)
-            utils_lib.run_cmd(self, cmd, expect_not_kw='deprecated,unsafe', msg='Check there is no "deprecated" or "unsafe" keyword in output from {}'.format(service))
+        utils_lib.run_cmd(self, 'cat {}'.format(self.systemd_analyze_verify_file),expect_ret=0, expect_not_kw='deprecated,unsafe', msg='Check there is no "deprecated" or "unsafe" keyword')
 
     def test_check_systemd_analyze_verify_missing(self):
         """
@@ -1256,16 +1162,7 @@ in cmdline as bug1859088")
             - service unit file content
             - output from 'systemd-analyze verify $service'
         """
-        cmd = "sudo systemd-analyze verify default.target"
-        utils_lib.run_cmd(self, cmd, expect_not_kw='is obsolet', msg='Check there is no obsolet keyword in output')
-        cmd = "systemctl list-unit-files |grep -v UNIT|grep -v listed|awk -F' ' '{print $1}'"
-        all_services = utils_lib.run_cmd(self, cmd, msg='retrive all systemd unit files').split('\n')
-
-        for service in all_services:
-            if not service or service.startswith('-'):
-                continue
-            cmd = "sudo systemd-analyze verify {}".format(service)
-            utils_lib.run_cmd(self, cmd, expect_not_kw='Missing,ignoring line', msg='Check there is no Missing keyword in output from {}'.format(service))
+        utils_lib.run_cmd(self, 'cat {}'.format(self.systemd_analyze_verify_file),expect_ret=0, expect_not_kw='Missing,ignoring line', msg='Check there is no Missing keyword in output')
 
     def test_check_systemd_analyze_verify_obsolete(self):
         '''
@@ -1290,16 +1187,7 @@ in cmdline as bug1859088")
         expected_result:
             No "is obsolete" found in output
         '''
-        cmd = "sudo systemd-analyze verify default.target"
-        utils_lib.run_cmd(self, cmd, expect_not_kw='is obsolet', msg='Check there is no obsolet keyword in output')
-        cmd = "systemctl list-unit-files |grep -v UNIT|grep -v listed|awk -F' ' '{print $1}'"
-        all_services = utils_lib.run_cmd(self, cmd, msg='retrive all systemd unit files').split('\n')
-
-        for service in all_services:
-            if not service or service.startswith('-'):
-                continue
-            cmd = "sudo systemd-analyze verify {}".format(service)
-            utils_lib.run_cmd(self, cmd, expect_not_kw='is obsolet', msg='Check there is no obsolet keyword in output from {}'.format(service))
+        utils_lib.run_cmd(self, 'cat {}'.format(self.systemd_analyze_verify_file),expect_ret=0, expect_not_kw='is obsolet', msg='Check there is no obsolet keyword in output')
 
     def test_check_systemd_analyze_verify_ordering_cycle(self):
         '''
@@ -1327,18 +1215,7 @@ in cmdline as bug1859088")
         '''
         cmd = 'sudo journalctl -b0'
         utils_lib.run_cmd(self, cmd, expect_not_kw='ordering cycle', msg='Check there is no ordering cycle in journal log')
-        cmd = "sudo systemd-analyze verify default.target"
-        utils_lib.run_cmd(self, cmd, expect_not_kw='ordering cycle', msg='Check there is no ordering cycle which may block boot up.')
-        #cmd = "sudo systemctl list-units --type target|grep target|awk -F' ' '{print $1}'"
-        cmd = "systemctl list-unit-files |grep -v UNIT|grep -v listed|awk -F' ' '{print $1}'"
-        all_services = utils_lib.run_cmd(self, cmd, msg='retrive all systemd unit files').split('\n')
-
-        for service in all_services:
-            if not service or service.startswith('-'):
-                continue
-            cmd = "sudo systemd-analyze verify {}".format(service)
-            utils_lib.run_cmd(self, cmd, expect_not_kw='ordering cycle', msg='Check there is no ordering cycle which may block boot up in {}'.format(service))
-
+        utils_lib.run_cmd(self, 'cat {}'.format(self.systemd_analyze_verify_file),expect_ret=0, expect_not_kw='ordering cycle', msg='Check there is no ordering cycle which may block boot up')
 
     def test_check_tsc_deadline_timer(self):
         '''
@@ -1450,8 +1327,8 @@ current_device"
         description:
             check the pkg differences with information from database
         key_steps:
-            1. # sudo rpm -qa
-            2. # sudo rpm -V $(pkg)
+            1. # rpm -qa
+            2. # sudo rpm -V $(pkgs)
         expected_result:
             No pkg differences with information from database.
         '''
@@ -1465,22 +1342,18 @@ current_device"
               'T':'mTime differs',
               'P':'caPabilities differ',
               '?':'test could not be performed'}
-        pkg=utils_lib.run_cmd(self, 'sudo rpm -qa', msg="list the pkgs")
-        pkg=pkg.split('\n')
         count=0
         differ=''
-        for i in range(len(pkg)-1):
-            out=utils_lib.run_cmd(self, f'sudo rpm -V {pkg[i]}', msg="check pkg differences")
-            if out != '':
-                out = out.split('\n')
-                if out[0][0] in '.S':
-                    count += 1
-                    differ += f'{count}.{pkg[i]}:\n'
-                    for j in out[:-1]:
-                        differ+=j+'\n'
-                        for k in j[0:9]:
-                            if k!='.':
-                                differ+=character[k]+'\n'
+        for line in self.output.split('\n'):
+            if line.startswith(('.','S')):
+                count += 1
+                pkg_file = line.split(' ')[-1]
+                pkg = utils_lib.run_cmd(self, 'sudo rpm -qf {}'.format(pkg_file), timeout=600)
+                differ += f'{count}.{pkg}\n'
+                differ+=line+'\n'
+                for k in line[0:9]:
+                    if k!='.' and k in character.keys():
+                        differ+=character[k]+'\n'
         self.log.info("{}".format(differ))
         allow_count = 25
         if count>=allow_count:
@@ -1518,23 +1391,20 @@ current_device"
         description:
             check pkg missing file problem
         key_steps:
-            1. # sudo rpm -qa
+            1. # rpm -qa
             2. # sudo rpm -V $(pkg)
         expected_result:
             No missing file found in output.
         '''
-        skip = ['rhui']
-        pkg = utils_lib.run_cmd(self, 'sudo rpm -qa', msg="list the pkgs")
-        pkg = pkg.split('\n')
-        for i in range(len(pkg) - 1):
-            status = 0
-            for j in skip:
-                if re.search(j, pkg[i]):
-                    status += 1
-                else:
-                    pass
-            if status == 0:
-                utils_lib.run_cmd(self, f'sudo rpm -V {pkg[i]}', expect_not_kw="missing", msg="check pkg missing problem")
+        count=0
+        differ=''
+        for line in self.output.split('\n'):
+            if 'missing' in line:
+                count += 1
+                pkg_file = line.split(' ')[-1]
+                pkg = utils_lib.run_cmd(self, 'sudo rpm -qf {}'.format(pkg_file), timeout=600)
+                differ += f'{count}.{pkg}:\n'
+                differ+=line+'\n'
 
     def test_collect_insights_result(self):
         '''
@@ -1566,10 +1436,10 @@ current_device"
         if not utils_lib.is_cmd_exist(self, cmd="insights-client"):
             self.skipTest('No insights-client installation found!')
         utils_lib.run_cmd(self,
-                    'sudo lscpu',
+                    'lscpu',
                     msg="get cpu information")
         utils_lib.run_cmd(self,
-                    'sudo rpm -q insights-client',
+                    'rpm -q insights-client',
                     msg="get insights-client version")
         utils_lib.run_cmd(self,
                     'sudo insights-client --register',
@@ -1606,7 +1476,7 @@ current_device"
             self.log.error("insights rule hit or other unexpected error")
 
     def tearDown(self):
-        self.log.info("{} test done".format(self.id()))
+        pass
 
 if __name__ == '__main__':
     unittest.main()
